package es.rickyepoderi.sample;

import java.security.Principal;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.security.auth.Subject;
import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.common.security.SimplePrincipal;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.interceptor.security.DefaultSecurityContext;
import org.apache.cxf.security.SecurityContext;
import org.apache.cxf.ws.security.wss4j.PolicyBasedWSS4JInInterceptor;
import org.apache.cxf.ws.security.wss4j.WSS4JInInterceptor;
import org.apache.wss4j.dom.WSConstants;
import org.apache.wss4j.dom.engine.WSSecurityEngineResult;
import org.apache.wss4j.dom.handler.WSHandlerConstants;
import org.apache.wss4j.dom.handler.WSHandlerResult;
import org.jboss.wsf.spi.deployment.Endpoint;
import static org.jboss.wsf.stack.cxf.Messages.MESSAGES;

/**
 * Example interceptor that gets the BinarySecurityToken sent in a signature and
 * uses it to call to the wildfly login module. The implementation gets the
 * previous result of a signature verification and with the certificate
 * associated to it performs a login against the security login module. It
 * follows the idea under <em>SubjectCreatingInterceptor.java</em>.
 *
 * @author rmartinc
 */
public class CertificateBinaryTokenInterceptor extends WSS4JInInterceptor {

    /**
     * Constructor.
     */
    public CertificateBinaryTokenInterceptor() {
        this(new HashMap<String, Object>());
    }

    /**
     * Constructor.
     *
     * @param properties
     */
    public CertificateBinaryTokenInterceptor(Map<String, Object> properties) {
        super(properties);
        getAfter().add(PolicyBasedWSS4JInInterceptor.class.getName());
    }

    /**
     * Get the certificate from the signature done in the results. Just get the
     * one that is result of a signature (action=2), is validated (the signature
     * was verified) and the certificate is attached.
     *
     * @param message The soap message with the verification results
     * @return The certificate associated to the signature
     */
    private X509Certificate getCertificate(SoapMessage message) {
        List<WSHandlerResult> handlerResults = (List<WSHandlerResult>) message.get(WSHandlerConstants.RECV_RESULTS);
        if (handlerResults != null) {
            for (WSHandlerResult handlerResult : handlerResults) {
                List<WSSecurityEngineResult> engResults = handlerResult.getResults();
                if (engResults != null) {
                    for (WSSecurityEngineResult engResult : engResults) {
                        Boolean validated = (Boolean) engResult.get(WSSecurityEngineResult.TAG_VALIDATED_TOKEN);
                        X509Certificate cert = (X509Certificate) engResult.get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
                        Integer action = (Integer) engResult.get(WSSecurityEngineResult.TAG_ACTION);
                        if (Boolean.TRUE.equals(validated) && cert != null && action != null && WSConstants.SIGN == action) {
                            return cert;
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * Just get the signature certificate and logs into wildfly using it. The
     * security domain should be configured to use a X509Certificate.
     *
     * @param message The message
     * @throws Fault Some error
     */
    @Override
    public void handleMessage(SoapMessage message) throws Fault {
        Endpoint ep = message.getExchange().get(Endpoint.class);
        X509Certificate cert = getCertificate(message);
        SecurityContext context = message.get(SecurityContext.class);
        if (cert == null || context == null || context.getUserPrincipal() == null) {
            // no certificate, do nothing
            super.handleMessage(message);
        } else {
            // certificate found => perform a login with it
            // the principal name is going to be as in the users.jks in lowercase
            Principal principal = new SimplePrincipal(cert.getSubjectX500Principal().getName().toLowerCase());
            Subject subject = new Subject();
            if (ep.getSecurityDomainContext().isValid(principal, cert, subject)) {
                ep.getSecurityDomainContext().pushSubjectContext(subject, principal, cert);
                SecurityContext sc = new DefaultSecurityContext(principal, subject);
                message.put(SecurityContext.class, sc);
            } else {
                throw MESSAGES.authenticationFailed(principal.getName());
            }
        }
    }
}
